// Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'dart:collection';

import 'ast.dart';
import 'block_parser.dart';
import 'document.dart';
import 'extension_set.dart';
import 'inline_parser.dart';

/// Converts the given string of Markdown to HTML.
/// 
/// * [readOnly] wether the checkbox is clickable.
String markdownToHtml(String markdown,
    {Iterable<BlockSyntax> blockSyntaxes,
    Iterable<InlineSyntax> inlineSyntaxes,
    ExtensionSet extensionSet,
    Resolver linkResolver,
    Resolver imageLinkResolver,
    bool inlineOnly: false, bool readOnly: true}) {
  var document = new Document(
      blockSyntaxes: blockSyntaxes,
      inlineSyntaxes: inlineSyntaxes,
      extensionSet: extensionSet,
      linkResolver: linkResolver,
      imageLinkResolver: imageLinkResolver,
      readOnly: readOnly);

  if (inlineOnly) return renderToHtml(document.parseInline(markdown));

  // Replace windows line endings with unix line endings, and split.
  var lines = markdown.replaceAll('\r\n', '\n').split('\n');

  return renderToHtml(document.parseLines(lines)) + '\n';
}

/// Renders [nodes] to HTML.
String renderToHtml(List<Node> nodes) => new HtmlRenderer().render(nodes);

typedef Iterable<String> _AttributesSorter(Iterable<String> names);

Iterable<String> _sort(Iterable<String> names)
=> (names.toList())..sort((a, b) => a.compareTo(b));

Iterable<String> _nosort(Iterable<String> names)
=> names;

/// Translates a parsed AST to HTML.
/// Unlike [HtmlRenderer], it doesn't generate extra linefeeds
/// to the output HTML.
class CondensedHtmlRenderer implements NodeVisitor {
  StringBuffer buffer;
  Set<String> uniqueIds;
  final _AttributesSorter _attrSorter;

  CondensedHtmlRenderer(): _attrSorter = _sort;
  CondensedHtmlRenderer.nosort(): _attrSorter = _nosort;

  String render(List<Node> nodes) {
    buffer = new StringBuffer();
    uniqueIds = new LinkedHashSet<String>();

    for (final node in nodes) node.accept(this);

    return buffer.toString();
  }

  void visitText(Text text) {
    buffer.write(text.text);
  }

  bool visitElementBefore(Element element) {
    buffer.write('<${element.tag}');

    for (var name in _attrSorter(element.attributes.keys)) {
      buffer.write(' $name="${element.attributes[name]}"');
    }

    // attach header anchor ids generated from text
    if (element.generatedId != null) {
      buffer.write(' id="${uniquifyId(element.generatedId)}"');
    }

    if (element.isEmpty) {
      // Empty element like <hr/>.
      buffer.write(' />');

      return false;
    } else {
      buffer.write('>');
      return true;
    }
  }

  void visitElementAfter(Element element) {
    buffer.write('</${element.tag}>');
  }

  /// Uniquifies an id generated from text.
  String uniquifyId(String id) {
    if (!uniqueIds.contains(id)) {
      uniqueIds.add(id);
      return id;
    }

    var suffix = 2;
    var suffixedId = '$id-$suffix';
    while (uniqueIds.contains(suffixedId)) {
      suffixedId = '$id-${suffix++}';
    }
    uniqueIds.add(suffixedId);
    return suffixedId;
  }
}

/// Translates a parsed AST to HTML.
/// Unlike [CondensedHtmlRenderer], it adds linefeeds before so-called block tags,
/// such as h1, h2 and so on. It improves the readability but it is hard to apply
/// `white-space: pre-wrap` and similar style.
class HtmlRenderer extends CondensedHtmlRenderer {
  static const _blockTags = const <String> {'blockquote','h1','h2','h3','h4','h5','h6','hr','p','pre'};

  HtmlRenderer();
  HtmlRenderer.nosort(): super.nosort();

  @override
  bool visitElementBefore(Element element) {
    // Hackish. Separate block-level elements with newlines.
    if (buffer.isNotEmpty && _blockTags.contains(element.tag)) {
      buffer.write('\n');
    }
    if (super.visitElementBefore(element)) {
      return true;
    } else {
      if (element.tag == 'br') {
        buffer.write('\n');
      }
      return false;
    }
  }
}
