// Copyright (c) 2011, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

import 'package:markd/markdown.dart';
import 'package:test/test.dart';

import 'util.dart';

void main() {
  testDirectory('original');

  // Block syntax extensions
  testFile('extensions/fenced_code_blocks.unit',
      blockSyntaxes: [const FencedCodeBlockSyntax()]);
  testFile('extensions/headers_with_ids.unit',
      blockSyntaxes: [const HeaderWithIdSyntax()]);
  testFile('extensions/setext_headers_with_ids.unit',
      blockSyntaxes: [const SetextHeaderWithIdSyntax()]);
  testFile('extensions/tables.unit', blockSyntaxes: [const TableSyntax()]);

  testFile('extensions/tables_ext.unit', blockSyntaxes: [const TableSyntax()]);

  // Inline syntax extensions
  testFile('extensions/emojis.unit', inlineSyntaxes: [new EmojiSyntax()]);
  testFile('extensions/inline_html.unit',
      inlineSyntaxes: [new InlineHtmlSyntax()]);

  group('Checkboxes 1', () {
    validateCore(
        'Checkboxes',
        '''
First.
* [ ] Item 1
* [x] Item 2
  * [ ]  Item 3
  * [ ] Item 4
    * [ ] Item 5
  * [ ] Item 6
* [ ] Item 7
  * [ ] Item 8
* [x] Item 9

Others
''',
        '''
<p>First.</p><ul><li class="todo"><input class="todo" data-line="1" disabled="disabled" type="checkbox" /> Item 1</li><li class="todo"><input checked="checked" class="todo" data-line="2" disabled="disabled" type="checkbox" /> Item 2<ul><li class="todo"><input class="todo" data-line="3" disabled="disabled" type="checkbox" />  Item 3</li><li class="todo"><input class="todo" data-line="4" disabled="disabled" type="checkbox" /> Item 4<ul><li class="todo"><input class="todo" data-line="5" disabled="disabled" type="checkbox" /> Item 5</li></ul></li><li class="todo"><input class="todo" data-line="6" disabled="disabled" type="checkbox" /> Item 6</li></ul></li><li class="todo"><input class="todo" data-line="7" disabled="disabled" type="checkbox" /> Item 7<ul><li class="todo"><input class="todo" data-line="8" disabled="disabled" type="checkbox" /> Item 8</li></ul></li><li class="todo"><input checked="checked" class="todo" data-line="9" disabled="disabled" type="checkbox" /> Item 9</li></ul>
<p>Others</p>
''');
    });

  group('Checkboxes', () {
    validateCore(
        'Checkboxes 2',
        '''
First.
1. [ ] Item 1
1. Item 2
    * [ ]  Item 3
    * [ ] Item 4
        * [x] Item 5
    * Item 6
1. [ ] Item 7
    * Item 8
1. Item 9

Others
''',
        '''
<p>First.</p><ol><li class="todo"><input class="todo" data-line="1" disabled="disabled" type="checkbox" /> Item 1</li><li>Item 2<ul><li class="todo"><input class="todo" data-line="3" disabled="disabled" type="checkbox" />  Item 3</li><li class="todo"><input class="todo" data-line="4" disabled="disabled" type="checkbox" /> Item 4<ul><li class="todo"><input checked="checked" class="todo" data-line="5" disabled="disabled" type="checkbox" /> Item 5</li></ul></li><li>Item 6</li></ul></li><li class="todo"><input class="todo" data-line="7" disabled="disabled" type="checkbox" /> Item 7<ul><li>Item 8</li></ul></li><li>Item 9</li></ol>
<p>Others</p>
''');
    });

  group('Corner cases', () {
    validateCore(
        'Incorrect Links',
        '''
5 Ethernet ([Music](
''',
        '''
<p>5 Ethernet ([Music](</p>
''');
    });

  group('Resolver', () {
    Node nyanResolver(String text, [_]) =>
        text.isEmpty ? null : new Text('~=[,,_${text}_,,]:3');
    validateCore(
        'simple link resolver',
        '''
resolve [this] thing
''',
        '''
<p>resolve ~=[,,_this_,,]:3 thing</p>
''',
        linkResolver: nyanResolver);

    validateCore(
        'simple image resolver',
        '''
resolve ![this] thing
''',
        '''
<p>resolve ~=[,,_this_,,]:3 thing</p>
''',
        imageLinkResolver: nyanResolver);

    validateCore(
        'can resolve link containing inline tags',
        '''
resolve [*star* _underline_] thing
''',
        '''
<p>resolve ~=[,,_*star* _underline__,,]:3 thing</p>
''',
        linkResolver: nyanResolver);

    validateCore(
        'link resolver uses un-normalized link label',
        '''
resolve [TH  IS] thing
''',
        '''
<p>resolve ~=[,,_TH  IS_,,]:3 thing</p>
''',
        linkResolver: nyanResolver);

    validateCore(
        'can resolve escaped brackets',
        r'''
resolve [\[\]] thing
''',
        '''
<p>resolve ~=[,,_[]_,,]:3 thing</p>
''',
        linkResolver: nyanResolver);

    validateCore(
        'can choose to _not_ resolve something, like an empty link',
        r'''
resolve [[]] thing
''',
        '''
<p>resolve ~=[,,_[]_,,]:3 thing</p>
''',
        linkResolver: nyanResolver);
  });

  group('Custom inline syntax', () {
    var nyanSyntax = <InlineSyntax>[new TextSyntax('nyan', sub: '~=[,,_,,]:3')];
    validateCore(
        'simple inline syntax',
        '''
nyan''',
        '''<p>~=[,,_,,]:3</p>
''',
        inlineSyntaxes: nyanSyntax);

    validateCore('dart custom links', 'links [are<foo>] awesome',
        '<p>links <a>are&lt;foo></a> awesome</p>\n',
        linkResolver: (String text, [_]) =>
            new Element.text('a', text.replaceAll('<', '&lt;')));

    // TODO(amouravski): need more tests here for custom syntaxes, as some
    // things are not quite working properly. The regexps are sometime a little
    // too greedy, I think.
  });

  group('Inline only', () {
    validateCore(
        'simple line',
        '''
        This would normally create a paragraph.
        ''',
        '''
        This would normally create a paragraph.
        ''',
        inlineOnly: true);
    validateCore(
        'strong and em',
        '''
        This would _normally_ create a **paragraph**.
        ''',
        '''
        This would <em>normally</em> create a <strong>paragraph</strong>.
        ''',
        inlineOnly: true);
    validateCore(
        'link',
        '''
        This [link](http://www.example.com/) will work normally.
        ''',
        '''
        This <a href="http://www.example.com/">link</a> will work normally.
        ''',
        inlineOnly: true);
    validateCore(
        'references do not work',
        '''
        [This][] shouldn't work, though.
        ''',
        '''
        [This][] shouldn't work, though.
        ''',
        inlineOnly: true);
    validateCore(
        'less than and ampersand are escaped',
        '''
        < &
        ''',
        '''
        &lt; &amp;
        ''',
        inlineOnly: true);
    validateCore(
        'keeps newlines',
        '''
        This paragraph
        continues after a newline.
        ''',
        '''
        This paragraph
        continues after a newline.
        ''',
        inlineOnly: true);
    validateCore(
        'ignores block-level markdown syntax',
        '''
        1. This will not be an <ol>.
        ''',
        '''
        1. This will not be an <ol>.
        ''',
        inlineOnly: true);
  });
}
